@@ -1,4 +1,4 @@
-from itertools import repeat, chain
+from itertools import repeat
 from typing import List, Optional, Dict
 
 from conditional import conditional
@@ -20,8 +20,7 @@ from model_clone.utils import (
 
 class CloneMixin(object):
 
-    """
-    CloneMixin mixin to duplicate an object using the model cls.
+    """CloneMixin mixin to duplicate an object using the model cls.
 
     :param _clone_fields: Restricted List of fields to copy from the instance.
     :type _clone_fields: collections.Iterable
@@ -93,10 +92,177 @@ class CloneMixin(object):
     USE_UNIQUE_DUPLICATE_SUFFIX = True  # type: bool
     MAX_UNIQUE_DUPLICATE_QUERY_ATTEMPTS = 100  # type: int
 
+    @classmethod
+    def check(cls, **kwargs):
+        errors = super(CloneMixin, cls).check(**kwargs)
+
+        if cls.USE_UNIQUE_DUPLICATE_SUFFIX and not cls.UNIQUE_DUPLICATE_SUFFIX:
+            errors.append(
+                Error(
+                    "UNIQUE_DUPLICATE_SUFFIX is required.",
+                    hint=(
+                        "Please provide UNIQUE_DUPLICATE_SUFFIX"
+                        + " for {} or set USE_UNIQUE_DUPLICATE_SUFFIX=False".format(
+                            cls.__name__
+                        )
+                    ),
+                    obj=cls,
+                    id="{}.E001".format(ModelCloneConfig.name),
+                )
+            )
+
+        if all([cls._clone_fields, cls._clone_excluded_fields]):
+            errors.append(
+                Error(
+                    "Conflicting configuration.",
+                    hint=(
+                        'Please provide either "_clone_fields"'
+                        + ' or "_clone_excluded_fields" for model {}'.format(
+                            cls.__name__
+                        )
+                    ),
+                    obj=cls,
+                    id="{}.E002".format(ModelCloneConfig.name),
+                )
+            )
+
+        if all([cls._clone_m2m_fields, cls._clone_excluded_m2m_fields]):
+            errors.append(
+                Error(
+                    "Conflicting configuration.",
+                    hint=(
+                        'Please provide either "_clone_m2m_fields"'
+                        + ' or "_clone_excluded_m2m_fields" for model {}'.format(
+                            cls.__name__
+                        )
+                    ),
+                    obj=cls,
+                    id="{}.E002".format(ModelCloneConfig.name),
+                )
+            )
+
+        if all(
+            [
+                cls._clone_m2o_or_o2m_fields,
+                cls._clone_excluded_m2o_or_o2m_fields,
+            ]
+        ):
+            errors.append(
+                Error(
+                    "Conflicting configuration.",
+                    hint=(
+                        "Please provide either "
+                        + '"_clone_m2o_or_o2m_fields"'
+                        + " or "
+                        + '"_clone_excluded_m2o_or_o2m_fields" for {}'.format(
+                            cls.__name__
+                        )
+                    ),
+                    obj=cls,
+                    id="{}.E002".format(ModelCloneConfig.name),
+                )
+            )
+
+        if all([cls._clone_o2o_fields, cls._clone_excluded_o2o_fields]):
+            errors.append(
+                Error(
+                    "Conflicting configuration.",
+                    hint=(
+                        'Please provide either "_clone_o2o_fields"'
+                        + ' or "_clone_excluded_o2o_fields" for {}'.format(cls.__name__)
+                    ),
+                    obj=cls,
+                    id="{}.E002".format(ModelCloneConfig.name),
+                )
+            )
+
+        return errors
+
+    @classmethod
+    def bulk_clone_multi(cls, objs, attrs=None, batch_size=None):
+        # type: (List[models.Model], Optional[List[Dict]], Optional[int]) -> List[models.Model]
+        # TODO: Support bulk clones split by the batch_szie
+        pass
+
+    @transaction.atomic
+    def make_clone(self, attrs=None, sub_clone=False):
+        """Creates a clone of the django model instance.
+
+        :param attrs: Dictionary of attributes to be replaced on the cloned object.
+        :type attrs: dict
+        :param sub_clone: Internal boolean used to detect cloning sub objects.
+        :type sub_clone: bool
+        :rtype: :obj:`django.db.models.Model`
+        :return: The model instance that has been cloned.
+        """
+        attrs = attrs or {}
+        if not self.pk:
+            raise ValidationError(
+                "{}: Instance must be saved before it can be cloned.".format(
+                    self.__class__.__name__
+                )
+            )
+        if sub_clone:
+            duplicate = self
+            duplicate.pk = None
+        else:
+            duplicate = self._create_copy_of_instance(self)
+            # Supports only updating the attributes of the base instance.
+            for name, value in attrs.items():
+                setattr(duplicate, name, value)
+
+        duplicate.full_clean()
+        duplicate.save()
+
+        duplicate = self.__duplicate_o2o_fields(duplicate)
+        duplicate = self.__duplicate_o2m_fields(duplicate)
+        duplicate = self.__duplicate_m2o_fields(duplicate)
+        duplicate = self.__duplicate_m2m_fields(duplicate, sub_clone)
+        return duplicate
+
+    def bulk_clone(self, count, attrs=None, batch_size=None, auto_commit=False):
+        ops = connections[self.__class__._default_manager.db].ops
+        objs = range(count)
+        clones = []
+        batch_size = batch_size or max(ops.bulk_batch_size([], list(objs)), 1)
+
+        with conditional(
+            auto_commit,
+            transaction_autocommit(using=self.__class__._default_manager.db),
+        ):
+            # If count exceeds the MAX_UNIQUE_DUPLICATE_QUERY_ATTEMPTS
+            with conditional(
+                self.MAX_UNIQUE_DUPLICATE_QUERY_ATTEMPTS < count,
+                context_mutable_attribute(
+                    self,
+                    "MAX_UNIQUE_DUPLICATE_QUERY_ATTEMPTS",
+                    count,
+                ),
+            ):
+                if not self.MAX_UNIQUE_DUPLICATE_QUERY_ATTEMPTS >= count:
+                    raise AssertionError(
+                        "An Unknown error has occurred: Expected ({}) >= ({})".format(
+                            self.MAX_UNIQUE_DUPLICATE_QUERY_ATTEMPTS, count
+                        ),
+                    )
+                clones = list(repeat(self.make_clone(attrs=attrs), batch_size))
+
+        return clones
+
+    def parallel_clone(self, count, attrs=None, batch_size=None, auto_commit=False):
+        # if this takes n time for t records
+        # t^n i.e 100 * 10ms = 1000ms to clone 100 objects.
+        # I'll like to reduce this down to max time to clone count/batch_size i.e
+        # If it take 100ms to clone 100 objects with a db of batch_size 100
+        # If it takes 10ms to clone 10 objects i'll like to keep this down to 10ms for
+        # max_num_of_threads i.e 10 threads for 100 objects.
+        # This should run in parallel
+        # Testing jit and cpython if they offer better API's.
+        pass
+
     @staticmethod
     def _create_copy_of_instance(instance, force=False, sub_clone=False):
-        """
-        Create a copy of an instance
+        """Create a copy of an instance
 
         :param instance: The instance to be duplicated.
         :type instance: `django.db.models.Model`
@@ -189,92 +355,6 @@ class CloneMixin(object):
 
         return new_instance
 
-    @classmethod
-    def check(cls, **kwargs):
-        errors = super(CloneMixin, cls).check(**kwargs)
-
-        if cls.USE_UNIQUE_DUPLICATE_SUFFIX and not cls.UNIQUE_DUPLICATE_SUFFIX:
-            errors.append(
-                Error(
-                    "UNIQUE_DUPLICATE_SUFFIX is required.",
-                    hint=(
-                        "Please provide UNIQUE_DUPLICATE_SUFFIX"
-                        + " for {} or set USE_UNIQUE_DUPLICATE_SUFFIX=False".format(
-                            cls.__name__
-                        )
-                    ),
-                    obj=cls,
-                    id="{}.E001".format(ModelCloneConfig.name),
-                )
-            )
-
-        if all([cls._clone_fields, cls._clone_excluded_fields]):
-            errors.append(
-                Error(
-                    "Conflicting configuration.",
-                    hint=(
-                        'Please provide either "_clone_fields"'
-                        + ' or "_clone_excluded_fields" for model {}'.format(
-                            cls.__name__
-                        )
-                    ),
-                    obj=cls,
-                    id="{}.E002".format(ModelCloneConfig.name),
-                )
-            )
-
-        if all([cls._clone_m2m_fields, cls._clone_excluded_m2m_fields]):
-            errors.append(
-                Error(
-                    "Conflicting configuration.",
-                    hint=(
-                        'Please provide either "_clone_m2m_fields"'
-                        + ' or "_clone_excluded_m2m_fields" for model {}'.format(
-                            cls.__name__
-                        )
-                    ),
-                    obj=cls,
-                    id="{}.E002".format(ModelCloneConfig.name),
-                )
-            )
-
-        if all(
-            [
-                cls._clone_m2o_or_o2m_fields,
-                cls._clone_excluded_m2o_or_o2m_fields,
-            ]
-        ):
-            errors.append(
-                Error(
-                    "Conflicting configuration.",
-                    hint=(
-                        "Please provide either "
-                        + '"_clone_m2o_or_o2m_fields"'
-                        + " or "
-                        + '"_clone_excluded_m2o_or_o2m_fields" for {}'.format(
-                            cls.__name__
-                        )
-                    ),
-                    obj=cls,
-                    id="{}.E002".format(ModelCloneConfig.name),
-                )
-            )
-
-        if all([cls._clone_o2o_fields, cls._clone_excluded_o2o_fields]):
-            errors.append(
-                Error(
-                    "Conflicting configuration.",
-                    hint=(
-                        'Please provide either "_clone_o2o_fields"'
-                        + ' or "_clone_excluded_o2o_fields" for {}'.format(cls.__name__)
-                    ),
-                    obj=cls,
-                    id="{}.E002".format(ModelCloneConfig.name),
-                )
-            )
-
-        return errors
-
     def __duplicate_o2o_fields(self, duplicate):
         """
         Duplicate the one to one fields.
@@ -283,15 +363,13 @@ class CloneMixin(object):
         :type duplicate: `django.db.models.Model`
         :return: The duplicate instance with all the one to one fields duplicated.
         """
-        if self._clone_o2o_fields or self._clone_excluded_o2o_fields:
-            for f in self._meta.related_objects:
+        for f in self._meta.related_objects:
+            if f.one_to_one:
                 if any(
                     [
-                        f.one_to_one
-                        and f.name in self._clone_o2o_fields
+                        f.name in self._clone_o2o_fields
                         and f not in self._meta.concrete_fields,
-                        f.one_to_one
-                        and self._clone_excluded_o2o_fields
+                        self._clone_excluded_o2o_fields
                         and f.name not in self._clone_excluded_o2o_fields
                         and f not in self._meta.concrete_fields,
                     ]
@@ -308,49 +386,73 @@ class CloneMixin(object):
 
         return duplicate
 
-    def __duplicate_o2m_m2o_fields(self, duplicate):
-        """
-        Duplicate many to one or one to many fields.
+    def __duplicate_o2m_fields(self, duplicate):
+        """Duplicate one to many fields.
 
         :param duplicate: The transient instance that should be duplicated.
         :type duplicate: `django.db.models.Model`
-        :return: The duplicate instance with all the many to one or one to many fields duplicated.
+        :return: The duplicate instance with all the one to many fields duplicated.
         """
         fields = set()
 
-        if self._clone_m2o_or_o2m_fields or self._clone_excluded_m2o_or_o2m_fields:
-            for f in self._meta.related_objects:
+        for f in self._meta.related_objects:
+            if f.one_to_many:
                 if any(
                     [
-                        any([f.many_to_one, f.one_to_many])
-                        and f.name in self._clone_m2o_or_o2m_fields,
-                        any([f.many_to_one, f.one_to_many])
+                        f.get_accessor_name() in self._clone_m2o_or_o2m_fields,
+                        self._clone_excluded_m2o_or_o2m_fields
+                        and f.get_accessor_name()
+                        not in self._clone_excluded_m2o_or_o2m_fields,
+                    ]
+                ):
+                    fields.add(f)
+
+        # Clone one to many fields
+        for field in fields:
+            for item in getattr(self, field.get_accessor_name()).all():
+                try:
+                    item.make_clone(attrs={field.remote_field.name: duplicate})
+                except IntegrityError:
+                    item.make_clone(
+                        attrs={field.remote_field.name: duplicate}, sub_clone=True
+                    )
+
+        return duplicate
+
+    def __duplicate_m2o_fields(self, duplicate):
+        """Duplicate many to one fields.
+
+        :param duplicate: The transient instance that should be duplicated.
+        :type duplicate: `django.db.models.Model`
+        :return: The duplicate instance with all the many to one fields duplicated.
+        """
+        fields = set()
+
+        for f in self._meta.concrete_fields:
+            if f.many_to_one:
+                if any(
+                    [
+                        f.name in self._clone_m2o_or_o2m_fields,
+                        self._clone_excluded_m2o_or_o2m_fields
                         and f.name not in self._clone_excluded_m2o_or_o2m_fields,
                     ]
                 ):
                     fields.add(f)
 
-        # Clone one to many/many to one fields
+        # Clone many to one fields
         for field in fields:
-            items = []
-            for item in getattr(self, field.related_name).all():
-                try:
-                    item_clone = item.make_clone(
-                        attrs={field.remote_field.name: duplicate}
-                    )
-                except IntegrityError:
-                    item_clone = item.make_clone(
-                        attrs={field.remote_field.name: duplicate}, sub_clone=True
-                    )
-                items.append(item_clone)
+            item = getattr(self, field.name)
+            try:
+                item_clone = item.make_clone()
+            except IntegrityError:
+                item_clone = item.make_clone(sub_clone=True)
 
-            getattr(duplicate, field.related_name).set(items)
+            setattr(duplicate, field.name, item_clone)
 
         return duplicate
 
     def __duplicate_m2m_fields(self, duplicate, sub_clone):
-        """
-        Duplicate many to many fields.
+        """Duplicate many to many fields.
 
         :param duplicate: The transient instance that should be duplicated.
         :type duplicate: `django.db.models.Model`
@@ -364,13 +466,23 @@ class CloneMixin(object):
         if sub_clone:
             return duplicate
 
-        if self._clone_m2m_fields or self._clone_excluded_m2m_fields:
-            for f in chain(self._meta.related_objects, self._meta.many_to_many):
+        for f in self._meta.many_to_many:
+            if any(
+                [
+                    f.name in self._clone_m2m_fields,
+                    self._clone_excluded_m2m_fields
+                    and f.name not in self._clone_excluded_m2m_fields,
+                ]
+            ):
+                fields.add(f)
+        for f in self._meta.related_objects:
+            if f.many_to_many:
                 if any(
                     [
-                        f.many_to_many and f.name in self._clone_m2m_fields,
-                        f.many_to_many
-                        and f.name not in self._clone_excluded_m2m_fields,
+                        f.get_accessor_name() in self._clone_m2m_fields,
+                        self._clone_excluded_m2m_fields
+                        and f.get_accessor_name()
+                        not in self._clone_excluded_m2m_fields,
                     ]
                 ):
                     fields.add(f)
@@ -381,8 +493,8 @@ class CloneMixin(object):
                 # ManyToManyRel
                 field_name = field.field.m2m_reverse_field_name()
                 through = field.through
-                source = getattr(self, field.related_name)
-                destination = getattr(duplicate, field.related_name)
+                source = getattr(self, field.get_accessor_name())
+                destination = getattr(duplicate, field.get_accessor_name())
             else:
                 through = field.remote_field.through
                 field_name = field.m2m_field_name()
@@ -406,85 +518,3 @@ class CloneMixin(object):
                 destination.set(source.all())
 
         return duplicate
-
-    @transaction.atomic
-    def make_clone(self, attrs=None, sub_clone=False):
-        """
-        Creates a clone of the django model instance.
-
-        :param attrs: Dictionary of attributes to be replaced on the cloned object.
-        :type attrs: dict
-        :param sub_clone: Internal boolean used to detect cloning sub objects.
-        :type sub_clone: bool
-        :rtype: :obj:`django.db.models.Model`
-        :return: The model instance that has been cloned.
-        """
-        attrs = attrs or {}
-        if not self.pk:
-            raise ValidationError(
-                "{}: Instance must be saved before it can be cloned.".format(
-                    self.__class__.__name__
-                )
-            )
-        if sub_clone:
-            duplicate = self
-            duplicate.pk = None
-        else:
-            duplicate = self._create_copy_of_instance(self)
-            # Supports only updating the attributes of the base instance.
-            for name, value in attrs.items():
-                setattr(duplicate, name, value)
-
-        duplicate.full_clean()
-        duplicate.save()
-
-        duplicate = self.__duplicate_o2o_fields(duplicate)
-        duplicate = self.__duplicate_o2m_m2o_fields(duplicate)
-        duplicate = self.__duplicate_m2m_fields(duplicate, sub_clone)
-        return duplicate
-
-    @classmethod
-    def bulk_clone_multi(cls, objs, attrs=None, batch_size=None):
-        # type: (List[models.Model], Optional[List[Dict]], Optional[int]) -> List[models.Model]
-        # TODO: Support bulk clones split by the batch_szie
-        pass
-
-    def bulk_clone(self, count, attrs=None, batch_size=None, auto_commit=False):
-        ops = connections[self.__class__._default_manager.db].ops
-        objs = range(count)
-        clones = []
-        batch_size = batch_size or max(ops.bulk_batch_size([], list(objs)), 1)
-
-        with conditional(
-            auto_commit,
-            transaction_autocommit(using=self.__class__._default_manager.db),
-        ):
-            # If count exceeds the MAX_UNIQUE_DUPLICATE_QUERY_ATTEMPTS
-            with conditional(
-                self.MAX_UNIQUE_DUPLICATE_QUERY_ATTEMPTS < count,
-                context_mutable_attribute(
-                    self,
-                    "MAX_UNIQUE_DUPLICATE_QUERY_ATTEMPTS",
-                    count,
-                ),
-            ):
-                if not self.MAX_UNIQUE_DUPLICATE_QUERY_ATTEMPTS >= count:
-                    raise AssertionError(
-                        "An Unknown error has occurred: Expected ({}) >= ({})".format(
-                            self.MAX_UNIQUE_DUPLICATE_QUERY_ATTEMPTS, count
-                        ),
-                    )
-                clones = list(repeat(self.make_clone(attrs=attrs), batch_size))
-
-        return clones
-
-    def parallel_clone(self, count, attrs=None, batch_size=None, auto_commit=False):
-        # if this takes n time for t records
-        # t^n i.e 100 * 10ms = 1000ms to clone 100 objects.
-        # I'll like to reduce this down to max time to clone count/batch_size i.e
-        # If it take 100ms to clone 100 objects with a db of batch_size 100
-        # If it takes 10ms to clone 10 objects i'll like to keep this down to 10ms for
-        # max_num_of_threads i.e 10 threads for 100 objects.
-        # This should run in parallel
-        # Testing jit and cpython if they offer better API's.
-        pass
